import { Component, ViewChild, HostBinding, Input, HostListener, EventEmitter, Output, forwardRef, Directive, ElementRef } from '@angular/core';
import { DropdownService } from '../dropdown/dropdown.service';
import { SuiDropdownMenu } from '../dropdown/dropdown-menu';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { SearchService } from './search.service';
import { readValue } from '../util/util';
var SuiSearch = (function () {
    /**
     * @param {?} _element
     */
    function SuiSearch(_element) {
        this._element = _element;
        this.dropdownService = new DropdownService();
        this.searchService = new SearchService();
        this._searchClasses = true;
        this.hasIcon = true;
        this.placeholder = "Search...";
        this.searchDelay = 200;
        this.onItemSelected = new EventEmitter();
    }
    Object.defineProperty(SuiSearch.prototype, "isActive", {
        /**
         * @return {?}
         */
        get: function () {
            return this.dropdownService.isOpen;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "query", {
        /**
         * @return {?}
         */
        get: function () {
            return this.searchService.query;
        },
        /**
         * @param {?} query
         * @return {?}
         */
        set: function (query) {
            var _this = this;
            this.selectedItem = null;
            // Initialise a delayed search.
            this.searchService.updateQueryDelayed(query, function () {
                // Set the results open state depending on whether a query has been entered.
                return _this.dropdownService.setOpenState(_this.searchService.query.length > 0);
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "options", {
        /**
         * @param {?} options
         * @return {?}
         */
        set: function (options) {
            if (typeof options == "function") {
                this.searchService.optionsLookup = options;
                return;
            }
            this.searchService.options = options;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "optionsField", {
        /**
         * @param {?} field
         * @return {?}
         */
        set: function (field) {
            this.searchService.optionsField = field;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "searchDelay", {
        /**
         * @param {?} delay
         * @return {?}
         */
        set: function (delay) {
            this.searchService.searchDelay = delay;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "isSearching", {
        /**
         * @return {?}
         */
        get: function () {
            return this.searchService.isSearching;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "results", {
        /**
         * @return {?}
         */
        get: function () {
            return this.searchService.results;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiSearch.prototype, "ngModelChange", {
        /**
         * @return {?}
         */
        get: function () {
            return this.onItemSelected;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    SuiSearch.prototype.ngAfterViewInit = function () {
        this._menu.service = this.dropdownService;
    };
    /**
     * @param {?} item
     * @return {?}
     */
    SuiSearch.prototype.select = function (item) {
        this.writeValue(item);
        this.onItemSelected.emit(item);
    };
    /**
     * @param {?} e
     * @return {?}
     */
    SuiSearch.prototype.onClick = function (e) {
        e.stopPropagation();
        if (this.searchService.query.length > 0) {
            // Only open on click when there is a query entered.
            this.dropdownService.setOpenState(true);
        }
    };
    /**
     * @param {?} object
     * @return {?}
     */
    SuiSearch.prototype.readValue = function (object) {
        return readValue(object, this.searchService.optionsField);
    };
    /**
     * @param {?} item
     * @return {?}
     */
    SuiSearch.prototype.writeValue = function (item) {
        this.selectedItem = item;
        this.searchService.updateQuery(item ? (this.readValue(item)) : "");
    };
    return SuiSearch;
}());
export { SuiSearch };
SuiSearch.decorators = [
    { type: Component, args: [{
                selector: 'sui-search',
                template: "\n<div class=\"ui input\" [class.icon]=\"hasIcon\">\n    <input class=\"prompt\" type=\"text\" [attr.placeholder]=\"placeholder\" autocomplete=\"off\" [(ngModel)]=\"query\">\n    <i *ngIf=\"hasIcon\" class=\"search icon\"></i>\n  </div>\n<div class=\"results\" suiDropdownMenu menuTransition=\"scale\" menuSelectedItemClass=\"active\">\n    <a class=\"result item\" *ngFor=\"let r of results\" (click)=\"select(r)\">\n        <span *ngIf=\"!searchService.optionsLookup\" [innerHTML]=\"searchService.highlightMatches(r)\"></span>\n        <span *ngIf=\"searchService.optionsLookup\">{{ readValue(r) }}</span>\n    </a>\n    <div *ngIf=\"results.length == 0\" class=\"message empty\">\n        <div class=\"header\">No Results</div>\n        <div class=\"description\">Your search returned no results.</div>\n    </div>\n</div>\n",
                styles: ["\n/* Ensures results div has margin. */\n:host {\n    display: inline-block;\n}\n\n/* Fixes positioning when results are pushed above the search. */\n.results {\n    margin-bottom: .5em;\n}\n"]
            },] },
];
/**
 * @nocollapse
 */
SuiSearch.ctorParameters = function () { return [
    { type: ElementRef, },
]; };
SuiSearch.propDecorators = {
    '_menu': [{ type: ViewChild, args: [SuiDropdownMenu,] },],
    '_searchClasses': [{ type: HostBinding, args: ['class.ui',] }, { type: HostBinding, args: ['class.search',] },],
    'isActive': [{ type: HostBinding, args: ['class.active',] },],
    'hasIcon': [{ type: Input },],
    'placeholder': [{ type: Input },],
    'options': [{ type: Input },],
    'optionsField': [{ type: Input },],
    'searchDelay': [{ type: Input },],
    'isSearching': [{ type: HostBinding, args: ['class.loading',] },],
    'onItemSelected': [{ type: Output },],
    'onClick': [{ type: HostListener, args: ["click", ['$event'],] },],
};
function SuiSearch_tsickle_Closure_declarations() {
    /** @type {?} */
    SuiSearch.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    SuiSearch.ctorParameters;
    /** @type {?} */
    SuiSearch.propDecorators;
    /** @type {?} */
    SuiSearch.prototype.dropdownService;
    /** @type {?} */
    SuiSearch.prototype.searchService;
    /** @type {?} */
    SuiSearch.prototype._menu;
    /** @type {?} */
    SuiSearch.prototype._searchClasses;
    /** @type {?} */
    SuiSearch.prototype.hasIcon;
    /** @type {?} */
    SuiSearch.prototype.placeholder;
    /** @type {?} */
    SuiSearch.prototype.selectedItem;
    /** @type {?} */
    SuiSearch.prototype.onItemSelected;
    /** @type {?} */
    SuiSearch.prototype._element;
}
// Value accessor for the search.
export var /** @type {?} */ SEARCH_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return SuiSearchValueAccessor; }),
    multi: true
};
var SuiSearchValueAccessor = (function () {
    /**
     * @param {?} host
     */
    function SuiSearchValueAccessor(host) {
        this.host = host;
        this.onChange = function () { };
        this.onTouched = function () { };
    }
    /**
     * @param {?} value
     * @return {?}
     */
    SuiSearchValueAccessor.prototype.writeValue = function (value) {
        this.host.writeValue(value);
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    SuiSearchValueAccessor.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    SuiSearchValueAccessor.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    return SuiSearchValueAccessor;
}());
export { SuiSearchValueAccessor };
SuiSearchValueAccessor.decorators = [
    { type: Directive, args: [{
                selector: 'sui-search',
                host: {
                    '(onItemSelected)': 'onChange($event)'
                },
                providers: [SEARCH_VALUE_ACCESSOR]
            },] },
];
/**
 * @nocollapse
 */
SuiSearchValueAccessor.ctorParameters = function () { return [
    { type: SuiSearch, },
]; };
function SuiSearchValueAccessor_tsickle_Closure_declarations() {
    /** @type {?} */
    SuiSearchValueAccessor.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    SuiSearchValueAccessor.ctorParameters;
    /** @type {?} */
    SuiSearchValueAccessor.prototype.onChange;
    /** @type {?} */
    SuiSearchValueAccessor.prototype.onTouched;
    /** @type {?} */
    SuiSearchValueAccessor.prototype.host;
}
//# sourceMappingURL=search.js.map